from __future__ import unicode_literals

import os
import uuid

from django.db import models
from django.contrib.auth.models import AbstractUser, Group

from django_date_extensions.fields import ApproximateDateField
from django.utils.safestring import mark_safe

from dcxml import simpledc

from taggit.managers import TaggableManager
from taggit.models import TagBase, GenericTaggedItemBase

from ddusers.cropped_thumbnail import cropped_thumbnail
from PIL import Image

from ddpublications.models import Publication, Publisher, Person, Place
from ddpublications.lookup_doi import lookup_doi

from django_comments.abstracts import CommentAbstractModel
from weasyprint import HTML
from django.template.loader import render_to_string


from django.forms import ModelForm
from django.urls import reverse

from django.contrib.contenttypes.fields import GenericRelation
from ddcomments.models import Comment




#from ckeditor.fields import RichTextField
from froala_editor.fields import FroalaField


ACCESS_RULES_CHOICES = (
    ('PUB', 'public'),
    ('NET', 'network'),
    ('GRP', 'group'),
    ('HID', 'hidden'),
)

ACADEMIC_TITLE_CHOICES = (
    ('PHD', 'PhD'),
    ('DR', 'Dr.'),
    ('PROF', 'Prof. Dr.'),
    ('MA', 'MA',),
    ('BA', 'BA',),
    ('MAG', 'Magister'),
    ('M.A.', 'M.A.'),
)

ACADEMIC_TITLE_CHOICES_ORDER = {
    'PHD': 'b',
    'DR': 'f',
    'PROF': 'f',
    'MA': 'b',
    'BA': 'b',
    'MAG':'f',
    'M.A.': 'b',
}


class EnTagged(TagBase):
    class Meta:
        verbose_name = "English Tag"
        verbose_name_plural = "English Tags"

class EnglishTags(GenericTaggedItemBase):
    tag = models.ForeignKey(EnTagged)

class AffiliationTagged(TagBase):
    class Meta:
        verbose_name = "Affiliation Tag"
        verbose_name_plural = "Affiliation Tags"

class AffiliationTags(GenericTaggedItemBase):
    tag = models.ForeignKey(AffiliationTagged, related_name="affiliation_tags")
    def __str__(self):
        return self.name

class KeywordTagged(TagBase):
    class Meta:
        verbose_name = "Keyword Tag"
        verbose_name_plural = "Keyword Tags"

class KeywordTags(GenericTaggedItemBase):
    tag = models.ForeignKey('KeywordTagged', related_name="keywordtags_keywordtagged")
    def __str__(self):
        return self.tag.name

class LanguageTagged(TagBase):
    class Meta:
        verbose_name = "Language Tag"
        verbose_name_plural = "Language Tags"

class LanguageTags(GenericTaggedItemBase):
    tag = models.ForeignKey('LanguageTagged', related_name="languagetags_languagetagged")
    def __str__(self):
        return self.tag.name

class User(AbstractUser):
    middle_name = models.CharField(max_length=200, blank=True)
    academic_title = models.CharField(max_length=5, choices=ACADEMIC_TITLE_CHOICES, blank='None') # deprecated
    name_prefix = models.CharField(max_length=200, blank=True, verbose_name='prefix', help_text='for example \"Dr\"')
    name_suffix = models.CharField(max_length=200, blank=True, verbose_name='suffix', help_text='for example \"PhD\"')
    #position = models.CharField(max_length=600)
    #institution = models.ForeignKey('Institution', blank=True, null=True, related_name='user_profile_institution')
    #institutions = models.ManyToManyField('Institution', blank=True,)
    image = models.ImageField(blank=True, null=True)
    #fields_of_interest = models.ManyToManyField('FOI', blank=True)
    topics = models.ManyToManyField('Topic', blank=True)
    interests = TaggableManager(blank=True, through=KeywordTags)
    receptive_tos = models.ManyToManyField('ReceptiveTo', blank=True)
    countries = models.ManyToManyField('Country', related_name='user_country', blank=True)
    research_profile_full = models.TextField(blank=True, verbose_name='Research Profile (full)')
    research_profile_short = models.TextField(max_length=280, blank=True, verbose_name='Research Profile (brief)', help_text='brief version, max. 280 characters')
    access = models.CharField(max_length=3, choices=ACCESS_RULES_CHOICES, default='PUB')
    publications = models.ManyToManyField('ddpublications.Publication', blank=True, related_name='user_publication')
    en_tags = TaggableManager(blank=True, through=EnglishTags)
    affiliation_tags = TaggableManager("affiliations", blank=True, through=AffiliationTags)
    external_profile = models.URLField(blank=True)

    class Meta:
        ordering = ['last_name']

    def get_academic_name(self):
        a_n = list()
        if self.name_prefix:
            a_n.append(self.name_prefix)

        if self.first_name:
            a_n.append(self.first_name)

        if self.middle_name:
            a_n.append(self.middle_name)

        if self.last_name:
            a_n.append(self.last_name)

        academic_name = ' '.join(a_n)

        if self.name_suffix:
            academic_name = academic_name + ", " + self.name_suffix

        return academic_name


    def get_countries(self):
        return self.countries.all().order_by('name')

    def get_interests(self):
        return self.interests.all().order_by('name')

    def get_main_affiliation(self):
        return Affiliation.objects.filter(user=self).filter(main=True)

    def get_affiliations(self):
        return Affiliation.objects.filter(user=self).order_by('-main')

    def get_datasets(self):
        return DataSet.objects.filter(user=self).order_by('title')

    def get_projects(self):
        return Project.objects.filter(user=self).order_by('name')
        #projects_list = list()
        #for project in Project.objects.filter(user=self):
        #    projects_list.append(project)
        #return projects_list

    def __str__(self):
        return self.get_academic_name()

    def save(self):
        super(User, self).save()

    # override for thumbnail generation
    def save(self, *args, **kwargs):
        #models.Model.save(self, *args, **kwargs)
        try:
            # in situ thumbnail generation at upload
            im = Image.open(self.image.path)
            thumbnail = cropped_thumbnail( im, [250,250] )
            thumbnail.save(self.image.path)
        except:
            # no image uploaded yet
            pass
        # Call the "real" save() method in the base class 'models.Model'.
        models.Model.save(self, *args, **kwargs)

class WebPage(models.Model):
    title = models.CharField(max_length=200)
    main_desc_heading = models.CharField(max_length=200)
    main_desc = models.TextField(blank=True)
    text_left_heading = models.CharField(max_length=200)
    text_left = models.TextField(blank=True)
    text_middle_heading = models.CharField(max_length=200)
    text_middle = models.TextField(blank=True)
    text_right_heading = models.CharField(max_length=200)
    text_right = models.TextField(blank=True)

    def __str__(self):
        return self.title

class ReceptiveTo(models.Model):
    text = models.CharField(max_length=200, blank=True)
    def __str__(self):
        return self.text

class Link(models.Model):
    url = models.URLField()
    doi = models.CharField(max_length=200, blank=True)
    description = models.CharField(max_length=400, blank=True)

    def lookup_doi(self):
        service, doi_dict = lookup_doi(self.doi)
        print( doi_dict['author'] )

    def __str__(self):
        return self.url



#class Publication(Publication):
#    class Meta:
#        proxy = True
#
#    #author = models.CharField(max_length=400,)
#    #title = models.CharField(max_length=400, )
#    #subtitle = models.CharField(max_length=400, blank=True)
#    #year = models.CharField(max_length=400, )
#    #included_in = models.CharField(max_length=400, )
#    #url = models.URLField(blank=True, null=True)
#    def full_harvard_citation(self):
#        #citation_list = list()
#
#        authors_list = list()
#        for given_name, family_name in self.authors_list_split:
#            author_str = ("%s, %s" % (family_name, given_name[:1]))
#            authors_list.append(author_str)
#
#        if len(self.authors_list) > 3:
#            authors_harvard = "%s et. al." % ( authors_list[0], )
#        if len(self.authors_list) == 3:
#            authors_harvard = "%s, %s, & %s" % (authors_list[0], authors_list[1], authors_list[2])
#        if len(self.authors_list) == 2:
#            authors_harvard = "%s & %s" % (authors_list[0], authors_list[1])
#        if len(self.authors_list) == 1:
#            authors_harvard = "%s" % (authors_list[0],)
#        if len(self.authors_list) == 0:
#            authors_harvard = "o.A."
#
#
#        citation_list = list()
#        citation_list.append(authors_harvard + " " + str(self.year))
#        if self.journal_or_book_title():
#            citation_list.append(self.title)
#            citation_list.append("<i>%s</i>" % (self.journal_or_book_title(),))
#            if self.volume:
#                citation_list.append(str(self.volume))
#            if self.number:
#                citation_list.append(str(self.number))
#        else:
#            citation_list.append("<i>%s</i>" % (self.title,))
#        if self.publisher:
#            citation_list.append(self.publisher)
#        if self.place:
#            citation_list.append(self.place)
#        if self.pages:
#            citation_list.append(self.pages)
#
#
#        #citation_list_clean = list()
#        #for elem in citation_list:
#        #    if elem != '' or elem != 'None':
#        #        citation_list_clean.append(elem)
#
#        return mark_safe( ", ".join(citation_list) )
#
#
#    def __str__(self):
#        #return self.title
#        return self.full_harvard_citation()


class ProtestEvent(models.Model):
    country = models.ForeignKey('Country', related_name='protest_event_country')
    date_from = ApproximateDateField()
    date_to = ApproximateDateField(blank=True)
    date_text = models.CharField(max_length=200, blank=True)
    duration = models.CharField(max_length=400, blank=True)
    cities = models.ManyToManyField('City', related_name='protest_event_city')
    city_desc = models.CharField(max_length=400, blank=True)
    location = models.CharField(max_length=400, blank=True )
    participants_nr = models.CharField(max_length=400, blank=True)
    claims = models.TextField()
    violent = models.BooleanField(default=True)
    treatment = models.TextField(blank=True)
    sources = models.TextField(blank=True)
    image = models.ImageField(blank=True, null=True)
    links = models.ManyToManyField('Link', blank=True)
    related_data = models.ManyToManyField('DataSet', related_name='protest_event_data_set', blank=True)
    related_projects = models.ManyToManyField('Project', related_name='protest_event_project', blank=True)

    def get_cities(self):
        cities = list()
        for city in self.cities.all():
            cities.append(city.name)
        return ','.join(cities)

    def __str__(self):
        return self.get_cities()

class City(models.Model):
    name = models.CharField(max_length=400)
    slug = models.SlugField()
    geo_lat = models.CharField(max_length=200, blank=True)
    geo_lng = models.CharField(max_length=200, blank=True)
    country = models.ForeignKey('Country', related_name='city_country')
    public = models.BooleanField(default=True)

    def __str__(self):
        return self.name


class Country(models.Model):
    name = models.CharField(max_length=200)
    slug = models.SlugField()
    geo_lat = models.CharField(max_length=200)
    geo_lng = models.CharField(max_length=200)
    public = models.BooleanField(default=True)

    class Meta:
        ordering = ['name']

    def __str__(self):
        return self.name

class FOI(models.Model):
    name = models.CharField(max_length=200)
    description = models.TextField(blank=True)
    is_country = models.BooleanField(default=False)

    def __str__(self):
        return self.title


class Topic(models.Model):
    title = models.CharField(max_length=200)
    description = models.TextField(blank=True)

    def __str__(self):
        return self.title

class Keyword(models.Model):
    title = models.CharField(max_length=200)
    description = models.TextField(blank=True)

    def __str__(self):
        return self.title

class Affiliation(models.Model):
    user = models.ForeignKey('User', related_name='affiliation_user')
    institution = models.ForeignKey('Institution', related_name='affiliation_institution', blank=True, null=True)
    name_of_institution = models.CharField(max_length=200)
    place_of_institution = models.CharField(max_length=200)
    country_of_institution = models.CharField(max_length=200)
    position = models.CharField(max_length=200)
    website_of_institution = models.URLField(blank=True)
    main = models.BooleanField(default=True)

    def __str__(self):
        return "%s, %s, %s, %s" % (self.position, self.name_of_institution, self.place_of_institution, self.country_of_institution)

class Institution(models.Model):
    name = models.CharField(max_length=600)
    original_name = models.CharField(max_length=600, blank=True)
    short_name = models.CharField(max_length=60, blank=True)
    department = models.CharField(max_length=600, blank=True)
    address = models.CharField(max_length=600)
    postcode = models.CharField(max_length=40, blank=True, null=True)
    image = models.ImageField(blank=True, null=True)
    image_large = models.ImageField(blank=True, null=True)
    website = models.URLField(blank=True)
    description = models.TextField(blank=True)
    short_description = models.CharField(max_length=400, blank=True)
    location = models.CharField(max_length=20, blank=True)
    city = models.CharField(max_length=120)
    country = models.ForeignKey('Country', related_name='institution_country', blank=True)
    public = models.BooleanField(default=True)
    admin = models.ForeignKey('User', related_name='institution_admin_user', blank=True, null=True)
    affiliated_users = models.ManyToManyField('User', blank=True, )

    def __str__(self):
        return self.name
    #def __str__(self):
    #    if self.department:
    #        return "%s: %s, %s (%s)" % (self.name, self.department, self.city, self.country)
    #    else:
    #        return "%s, %s (%s)" % (self.name, self.city, self.country)

class Project(models.Model):
    user = models.ForeignKey('User', related_name='project_user')
    group = models.ForeignKey(Group, blank=True, null=True, related_name='project_group')
    name = models.CharField(max_length=200)
    url = models.URLField(blank=True)

    def __str__(self):
        return self.name

class DataFile(models.Model):
    DATA_FILE_TYPES = (
        ('DAT', 'data'),
        ('MET', 'metadata'),
        ('DOC', 'data documentation'),
        ('CFF', 'converted file format'),
    )
    dataset = models.ForeignKey('DataSet', related_name='datafile_dataset_dataset')
    file = models.FileField()#upload_to='datasets/')
    data_file_type = models.CharField(max_length=3, choices=DATA_FILE_TYPES, default='DAT',)
    data_file_format = models.CharField(max_length=200, default="unknown")
    data_file_size = models.IntegerField(default=1)
    content_type = models.CharField(max_length=200, default="text/plain")
    name = models.CharField(max_length=200,)
    repository_file_id = models.CharField(max_length=200, default='not set')
    repository = models.CharField(max_length=200, default="dariah-repository")

    def extension(self):
        name, extension = os.path.splitext(self.file.name)
        return extension

    def get_user(self):
        return self.dataset.user

    def get_group(self):
        return self.dataset.group

    def __str__(self):
        return self.name

    class Meta:
        permissions = (
            ('view_datafile', 'View Datafile'),
        )

class Category(models.Model):
    name = models.CharField(max_length=400)
    slug = models.CharField(max_length=400)
    image = models.ImageField(blank=True, null=True)
    description = models.TextField()
    sponsors = models.ManyToManyField("Sponsor", blank=True, related_name='category_sponsors_sponsor')
    curators = models.ManyToManyField("User", blank=True, related_name="category_curators_user")

    def __str__(self):
        return self.name


class Sponsor(models.Model):
    name = models.CharField(max_length=400)
    institution = models.ForeignKey('Institution', blank=True, null=True, related_name='sponsor_institution')
    logo = models.ImageField(blank=True, null=True)
    url = models.URLField(blank=True)

    def __str__(self):
        return self.name

class DataRepository(models.Model):
    name = models.CharField(max_length=400)
    logo = models.ImageField(blank=True, null=True)

    def __str__(self):
        return self.name

class DataType(models.Model):
    name = models.CharField(max_length=400)
    logo = models.ImageField(blank=True, null=True)

    def __str__(self):
        return self.name

class License(models.Model):
    name = models.CharField(max_length=400)
    logo = models.ImageField(blank=True, null=True)

    def __str__(self):
        return self.name

class CollectionMethodsTagged(TagBase):
    class Meta:
        verbose_name = "Collection Methods Tag"
        verbose_name_plural = "Collection Methods Tags"

class CollectionMethodsTags(GenericTaggedItemBase):
    tag = models.ForeignKey('CollectionMethodsTagged', related_name="collectionmethodstags_collectionmethodstagged")
    def __str__(self):
        return self.tag.name

class AnalysisMethodsTagged(TagBase):
    class Meta:
        verbose_name = "Analysis Methods Tag"
        verbose_name_plural = "Analysis Methods Tags"

class AnalysisMethodsTags(GenericTaggedItemBase):
    tag = models.ForeignKey('AnalysisMethodsTagged', related_name="analysismethodstags_analysismethodstagged")
    def __str__(self):
        return self.tag.name

class Documentation(models.Model):
    DOCU_TYPES = (
        ('GEN', 'general documentation'),
        ('LEG', 'legal documents'),
        ('BPR', 'best practices'),
        ('USE', 'user documentation'),
    )
    title = models.CharField(max_length=200)
    slug = models.CharField(max_length=200, help_text="Text used for URL generation, please use '_' between words")
    subtitle = models.CharField(max_length=200, blank=True)
    docu_type = models.CharField(max_length=3, choices=DOCU_TYPES, default='GEN',)
    file = models.FileField(blank=True, null=True)  # upload_to='datasets/')
    description = models.TextField(max_length=280, blank=True)
    text = models.TextField(blank=True)
    public = models.BooleanField(default=False)

    def __str__(self):
        return self.title


class DataSetPublication(models.Model):
    PUB_TYPES = (
        ('DIS', 'discusses dataset'),
        ('USE', 'uses dataset'),
        ('DES', 'describes dataset'),
        #('MET', 'similiar methodology'),
        #('SIM', 'similiar approach'),
    )
    dataset = models.ForeignKey('DataSet', on_delete=models.CASCADE)
    publication = models.ForeignKey('ddpublications.Publication', on_delete=models.CASCADE, null=True)
    pub_type = models.CharField(max_length=3, choices=PUB_TYPES, default='DIS', verbose_name='Type of publication', help_text='Relation between dataset and publication' )

    def __str__(self):
        return mark_safe("%s (%s)" % (self.publication, self.get_pub_type_display()))


class DataSet(models.Model):
    user = models.ForeignKey(User)
    group = models.ForeignKey(Group)
    institution = models.ForeignKey('Institution', blank=True, null=True, related_name='dataset_institution')
    title = models.CharField(max_length=200)
    subtitle = models.CharField(max_length=200, blank=True)
    categories = models.ManyToManyField("Category", related_name='dataset_categories_category', blank=True)
    image = models.ImageField(blank=True, null=True)
    countries = models.ManyToManyField(Country, related_name='dataset_country', blank=True)
    #access = models.CharField(max_length=3, choices=ACCESS_RULES_CHOICES, default='NET')
    link = models.ForeignKey('Link', blank=True,null=True, related_name='dataset_link')
    creators = models.CharField(max_length=400, help_text="Person(s) or institution(s) who authored the data set")
    data_repository = models.ForeignKey('DataRepository', blank=True, null=True, related_name='dataset_datarepository_data_repository')
    publication_date = models.DateField(auto_now_add=True, null=True)
    date_of_data_creation_from = models.DateField(blank=True, null=True, help_text="Date format YYYY-MM-DD", verbose_name="Date/period of data creation (from)")
    date_of_data_creation_to = models.DateField(blank=True, null=True, help_text="Date format YYYY-MM-DD", verbose_name="Date/period of data creation (to)")
    date_of_data_creation_text = models.CharField(max_length=400, help_text="If you cannot specify the exact date please indicate the approximate date here (eq. \"October 2010\")", blank="True")
    version = models.DecimalField(max_digits=5, decimal_places=1,default=1.0)
    datatypes = models.ManyToManyField('DataType', related_name='dataset_datatypes_datatype')
    description = models.TextField(verbose_name='Technical description')
    keyword_tags = TaggableManager("KeywordTags", blank=True, through=KeywordTags)
    sources_of_data = models.TextField()
    languages_of_data = TaggableManager("LanguageTags", blank=True, through=LanguageTags)
    methods_of_data_collection = TaggableManager("CollectionMethodsTags", blank=True, through=CollectionMethodsTags)
    methods_of_data_analysis = TaggableManager("AnalysisMethodsTags", blank=True, through=AnalysisMethodsTags)
    time_period_text = models.CharField(max_length=400, blank=True, help_text="If you cannot specify the exact date please indicate the approximate date here (eq. \"October 2010\")", verbose_name="Time period covered (text)")
    time_period_from = models.DateField(blank=True, null=True, help_text="Date format YYYY-MM-DD", verbose_name="Time period covered in data collection (from)")
    time_period_to = models.DateField(blank=True, null=True, help_text="Date format YYYY-MM-DD", verbose_name="Time period covered in data collection (to)")
    license = models.ForeignKey('License', blank=True, null=True, related_name='dataset_license_license')
    related_dataset_text = models.TextField(blank=True)
    related_dataset = models.ManyToManyField('DataSet', related_name='dataset_related_dataset_dataset', blank=True, help_text='Undo choice with CTRL–Mouse Click')
    #context = models.TextField(blank=True,)
    institutional_affiliation = models.TextField(blank=True,)
    sponsors = models.ManyToManyField('Sponsor', blank=True, related_name='dataset_sponsors')
    funding = models.TextField(blank=True,)
    publications = models.ManyToManyField('ddpublications.Publication', blank=True, through='DataSetPublication')
    comments = GenericRelation(Comment, related_query_name='comments')
    #comments = models.TextField(blank=True,)
    #html_string = models.TextField(blank=True)

    class Meta:
        permissions = (
            ('view_dataset', 'View Data Set'),
        )

    def generate_html_string(self):
        class PDFForm(ModelForm):
            class Meta:
                model = DataSet
                exclude =('html_string', 'user', 'group', 'institution', 'title', 'title_en', 'title_ru', 'subtitle', 'subtitle_en', 'subtitle_ru', 'image', 'categories', 'countries', 'description')

        form = PDFForm(instance=self)
        html_string = render_to_string('data/pdf_dataset.html', {'fields': form, 'ds': self, } )

        return html_string

    def get_users(self):
        # strange import to avoid getting caught in circular import triggerd by guardian; conflict because guardian uses User Model defined in this file
        # TODO: define User model in seperate file
        from .views import dataset_get_users
        return dataset_get_users(self)

    def get_comments(self):
        # return comments in reverse date order
        return self.comments.order_by('date_added')

    def get_pdf_file_name(self):
        return "DiscussData-%s-description.pdf" % (self.title.replace(' ', '_'))

    def get_zip_file_name(self):
        return "DicussData-%s-all_files.zip" % (self.title.replace(' ', '_'))

    #def generate_pdf(self):
    #    html = HTML(string=self.html_string, base_url=request.build_absolute_uri())
    #    file_name = "/tmp/dataset_%s.pdf" % (self.id, )
    #    return html.write_pdf()

    def get_absolute_url(self):
        abs = reverse('ddusers.dataset_detail', args=[str(self.id)])
        prefix = "https://dev.discuss-data.net"
        return prefix + abs

    def get_fulltitle(self):
        if self.subtitle:
            fulltitle = "%s – %s" % (self.title, self.subtitle)
        else:
            fulltitle = self.title
        return fulltitle

    def get_publications(self):
        return self.publications.all()

    def get_dataset_publications(self):
        return DataSetPublication.objects.filter(dataset=self)

    def get_countries(self):
        return self.countries.all()

    def get_keyword_tags(self):
        return self.keyword_tags.all()

    def get_language_tags(self):
        return self.languages_of_data.all()

    def get_methods_of_data_collection_tags(self):
        return self.methods_of_data_collection.all()

    def get_methods_of_data_analysis_tags(self):
        return self.methods_of_data_analysis.all()


    def get_data_citation(self):
        citation ="%s: %s (%s), Disuss Data, Version %s, <a href='%s'>%s</a>" \
                  % (self.creators, self.title, self.publication_date.year, self.version, self.get_absolute_url(), self.get_absolute_url())
        """
        This citation has seven components of which five are human readable:
        the author(s), title, year, data repository (or distributor), and version number.
        Two components are machine-readable (UNF = universal numerical fingerprint; DOI = Digital Object Identifier;
        ‘hdl’ refers to the international HANDLE.NET system).
        """
        return mark_safe(citation)

    def get_data_citation_pdf(self):
        citation ="%s: %s (%s), Disuss Data, Version %s, http://discuss-data.net/users/dataset/%s" \
                  % (self.creators, self.title, self.publication_date.year, self.version, self.id)
        return mark_safe(citation)

    # create new group on save
    def save(self, *args, **kwargs):
        try:
            self.group
        except:
            group_name = str(uuid.uuid4())
            new_group = Group.objects.create(name=group_name)
            self.group = new_group
            self.user.groups.add(self.group)
        super(DataSet, self).save(*args, **kwargs)
        models.Model.save(self, *args, **kwargs)
        #self.write_pdf()
        #self.html_string = self.generate_html_string()
        try:
            # in situ thumbnail generation at upload
            im = Image.open(self.image.path)
            thumbnail = cropped_thumbnail( im, [250,250] )
            thumbnail.save(self.image.path)
        except:
            # no image uploaded yet
            pass
        # Call the "real" save() method in the base class 'models.Model'.
        models.Model.save(self, *args, **kwargs)


    #def get_description(self):
    #    return DataSetDescription.objects.get(dataset=self)

    def get_datafiles(self):
        return DataFile.objects.filter(dataset=self)


    def dublin_core(self):
        dc_dict = dict(
            titles = [self.title_en, self.subtitle_en],
            #creators = [self.get_owners()],
        )
        #dc_dict['titles'] =  "<dc:title>%s</dc:title>\n" % (self.title,)
        #dc_str += "<dc:subtitle>%s</dc:subtitle>\n" % (self.subtitle,)
        return simpledc.tostring(dc_dict)

    def __str__(self):
        return self.title


